@@ -54,6 +54,13 @@ typedef struct {
 	 * Must not emit signals or destroy objects while holding it.
 	 */
 	GMutex *host_lock;
+
+	GMainContext *async_context;
+
+	/* Holds the timeout value for the connection, when
+	   no response is received.
+	*/
+	guint timeout;
 } SoupSessionPrivate;
 #define SOUP_SESSION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SOUP_TYPE_SESSION, SoupSessionPrivate))
 
@@ -94,6 +101,8 @@ enum {
 	PROP_MAX_CONNS_PER_HOST,
 	PROP_USE_NTLM,
 	PROP_SSL_CA_FILE,
+	PROP_ASYNC_CONTEXT,
+	PROP_TIMEOUT,
 
 	LAST_PROP
 };
@@ -116,6 +125,8 @@ soup_session_init (SoupSession *session)
 
 	priv->max_conns = SOUP_SESSION_MAX_CONNS_DEFAULT;
 	priv->max_conns_per_host = SOUP_SESSION_MAX_CONNS_PER_HOST_DEFAULT;
+
+	priv->timeout = 0;
 }
 
 static gboolean
@@ -128,9 +139,15 @@ foreach_free_host (gpointer key, gpointer host, gpointer data)
 static void
 cleanup_hosts (SoupSessionPrivate *priv)
 {
+	GHashTable *old_hosts;
+
 	g_mutex_lock (priv->host_lock);
-	g_hash_table_foreach_remove (priv->hosts, foreach_free_host, NULL);
+	old_hosts = priv->hosts;
+	priv->hosts = g_hash_table_new (host_uri_hash, host_uri_equal);
 	g_mutex_unlock (priv->host_lock);
+
+	g_hash_table_foreach_remove (old_hosts, foreach_free_host, NULL);
+	g_hash_table_destroy (old_hosts);
 }
 
 static void
@@ -165,6 +182,9 @@ finalize (GObject *object)
 	g_hash_table_destroy (priv->hosts);
 	g_hash_table_destroy (priv->conns);
 
+	if (priv->async_context)
+		g_main_context_unref (priv->async_context);
+
 	G_OBJECT_CLASS (soup_session_parent_class)->finalize (object);
 }
 
@@ -307,6 +327,19 @@ soup_session_class_init (SoupSessionClass *session_class)
 				     "File containing SSL CA certificates",
 				     NULL,
 				     G_PARAM_READWRITE));
+	g_object_class_install_property (
+		object_class, PROP_ASYNC_CONTEXT,
+		g_param_spec_pointer (SOUP_SESSION_ASYNC_CONTEXT,
+				      "Async GMainContext",
+				      "The GMainContext to dispatch async I/O in",
+				      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	g_object_class_install_property (
+		object_class, PROP_TIMEOUT,
+		g_param_spec_uint (SOUP_SESSION_TIMEOUT,
+				   "Timeout value",
+				   "Value in seconds to timeout a blocking I/O",
+				   0, G_MAXUINT, 0,
+				   G_PARAM_READWRITE));
 }
 
 static void
@@ -397,6 +430,14 @@ set_property (GObject *object, guint prop_id,
 			cleanup_hosts (priv);
 		}
 
+		break;
+	case PROP_ASYNC_CONTEXT:
+		priv->async_context = g_value_get_pointer (value);
+		if (priv->async_context)
+			g_main_context_ref (priv->async_context);
+		break;
+	case PROP_TIMEOUT:
+		priv->timeout = g_value_get_uint (value);
 		break;
 	default:
 		break;
@@ -428,6 +469,12 @@ get_property (GObject *object, guint prop_id,
 	case PROP_SSL_CA_FILE:
 		g_value_set_string (value, priv->ssl_ca_file);
 		break;
+	case PROP_ASYNC_CONTEXT:
+		g_value_set_pointer (value, priv->async_context ? g_main_context_ref (priv->async_context) : NULL);
+		break;
+	case PROP_TIMEOUT:
+		g_value_set_uint (value, priv->timeout);
+		break;
 	default:
 		break;
 	}
@@ -1142,6 +1189,8 @@ soup_session_get_connection (SoupSession *session, SoupMessage *msg,
 		SOUP_CONNECTION_PROXY_URI, priv->proxy_uri,
 		SOUP_CONNECTION_SSL_CREDENTIALS, priv->ssl_creds,
 		SOUP_CONNECTION_MESSAGE_FILTER, session,
+		SOUP_CONNECTION_ASYNC_CONTEXT, priv->async_context,
+		SOUP_CONNECTION_TIMEOUT, priv->timeout,
 		NULL);
 	g_signal_connect (conn, "connect_result",
 			  G_CALLBACK (connect_result),
